unit BitStream;

interface
 uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;


  Const
    SYNC_START_CODE = $000001;
  	PIC_START_CODE  = $00000100;
	  SLICE_MIN_CODE  = $00000101;
  	SLICE_MAX_CODE  = $000001af;
	  USER_START_CODE = $000001b2;
	  SEQ_START_CODE  = $000001b3;
	  EXT_START_CODE  = $000001b5;
   	SEQ_END_CODE    = $000001b7;
  	GOP_START_CODE  = $000001b8;

  type
   TBitsStream=class
   private
     bitbuffer:integer;
     bitcount:integer;
     buffer:array[0..1024-1] of integer;
     count:integer;
     position:integer;
     Stream:TMemoryStream;
     function get32Bits():integer;
     function show32Bits():integer;
     function read32Bits():integer;
   public
     Constructor Create(FileName:String);
     Destructor Destroy; override;
     function getCode():integer;
     function showCode():integer;
     function getBits( nbits:integer):integer;
     function showBits(nbits:integer):integer;
     procedure flushBits(nbits:integer);
     procedure alignBits(nbits:integer);

   End;

implementation

constructor TBitsStream.Create(FileName:String);
begin
    stream := TMemoryStream.Create;
    stream.LoadFromFile(FileName);
    stream.Position:=0;
    bitbuffer := 0;
    bitcount := 0;
		count := 0;
    position := 0;
end;

destructor TBitsStream.Destroy;
begin
  stream.Free;
  inherited;
end;

function TBitsStream.getCode():integer;
 begin
  alignBits(8);
		while (showBits(24) <> SYNC_START_CODE) do
    flushBits(8);
	result:= getBits(32);
end;

function TBitsStream.showCode():integer;
begin
  alignBits(8);
		while (showBits(24) <> SYNC_START_CODE) do
    flushBits(8);
  result:= showBits(32);
end;

function TBitsStream.getBits( nbits:integer):integer;
var
 bits:integer;
begin
   if (nbits <= bitcount) then begin
			bits := bitbuffer shr (32 - nbits);
			bitbuffer :=bitbuffer shl nbits;
			dec(bitcount,nbits);
		end else begin
			bits := bitbuffer shr (32 - nbits);
			dec(nbits,bitcount);
			bitbuffer := get32Bits();
			bits :=bits or (bitbuffer shr (32 - nbits));
			bitbuffer :=	bitbuffer shl nbits;
			bitcount := 32 - nbits;
		end;
		if (nbits >= 32) then
			  bitbuffer := 0;
		result:= bits;
end;

function TBitsStream.showBits(nbits:integer):integer;
var
 bits:integer;
begin
  bits := bitbuffer shr (32 - nbits);
		if (nbits > bitcount) then begin
			bits :=bits or (show32Bits() shr (32 + bitcount - nbits));
		end;
  result:= bits;
end;

procedure TBitsStream.flushBits(nbits:integer);
begin
   if (nbits <= bitcount)then begin
			bitbuffer :=bitbuffer shl nbits;
			dec(bitcount ,nbits);
		end else begin
			dec(nbits,bitcount);
			bitbuffer := get32Bits() shl nbits;
			bitcount:= 32 - nbits;
		end;
end;

procedure TBitsStream.alignBits(nbits:integer);
begin
		flushBits(bitcount mod nbits);
end;

function TBitsStream.get32Bits():integer;
begin
		if (position >= count) then begin
			position := 0;
      count := 0;
			while count < length(buffer) do begin
				buffer[count] := read32Bits();
        inc(count);
        end;
		end;
   result:= buffer[position];
   inc(position);
end;

function TBitsStream.show32Bits():integer;
begin
		if (position >= count) then begin
			position := 0;
      count := 0;
			while count < length(buffer) do begin
				buffer[count] := read32Bits();
        inc(count);
        end;
		end;
		result:= buffer[position];
end;

function TBitsStream.read32Bits():integer;
var
bitbuf:array[0..3] of byte;
begin
    	if (stream.Position > stream.Size) then begin
			result:= SEQ_END_CODE;
      exit;
      end;
  stream.Read(bitbuf,4);
  result:=(bitbuf[0] shl 24) or (bitbuf[1] shl 16) or (bitbuf[2] shl 8) or (bitbuf[3] shl 0);
end;

end.
